<?php
/**
 * @author Zoltán Zvara
 * @copyright Zoltán Zvara, 2014
 * @package Control
 */
namespace Arylium\Control;

use Arylium\Component;
use Arylium\Site;

/**
 * Thrown, when handler has not been found for the specified action,
 * neither default handler has been defined.
 * @package Arylium\Control
 */
class Unhandled extends Exception
{
}

/**
 * Being a part of the control means the class implements the
 * following interface.
 */
abstract class Control extends Component
{
    /**
     * @todo Document.
     * @var Array
     */
    protected $Handlers;
    /**
     * Database connection.
     */
    protected $Database;
    /**
     * Displayer.
     * @var \View\Displayer
     */
    protected $Displayer;
    /**
     * Site and request specific data.
     * @var Site
     */
    protected $Site;
    /**
     * Namespace given by the router.
     * @var string
     */
    protected $namespace;
    /**
     * Service name found by the router.
     * @var string
     */
    protected $service;
    /**
     * Action name found by the router.
     * @var string
     */
    protected $action;
    /**
     * Request parameters fetched by the router.
     * @var Array
     */
    protected $Parameters = [];

    /**
     * Constructs and initializes a default controller.
     * @param $Parameters
     * @param string $service
     * @param string $namespace
     * @param string $action
     * @returns Control
     * @todo Add session management.
     * @todo Add client management.
     */
    public function __construct($Parameters, $service, $namespace, $action = NULL)
    {
        parent::__construct();

        $this->service = $service;
        $this->namespace = $namespace;
        $this->action = $action;
        $this->Parameters = $Parameters;

        try {
            $this->Site = new Site($service);
        } catch (Exception $exception) {
            $this->abandon($exception);
        }
    }

    /**
     * Starts the serving process of the user request.
     */
    public function serve()
    {
        try {
            $this->handle();
        } catch (Exception $exception) {
            try {
                $this->beforeAbandon($exception);
            } catch (Exception $exception) {
                $this->abandon($exception);
            }
        }
    }

    /**
     * Navigates (routes) through, chooses a function and completes a request.
     */
    protected function handle()
    {
        // If there is a method for that action.
        if (method_exists($this, $this->action)) {
            call_user_func_array([self::class, $this->action], $this->Parameters);
            // If there is a method specified in the handlers.
        } else if (array_key_exists($this->action, $this->Handlers)
            && method_exists($this, $this->Handlers[$this->action])
        ) {
            call_user_func_array([self::class, $this->Handlers[$this->action]], $this->Parameters);
            // Else call the main handler.
        } else {
            $this->main();
        }
    }

    /**
     * Any fatal exception handling should happen here.
     */
    abstract protected function abandon(Exception $exception);

    /**
     * Runs before the abandon function.
     */
    protected function beforeAbandon(Exception $exception)
    {
        throw $exception;
    }

    /**
     * Main handler for actions.
     * @throws Unhandled
     */
    protected function main()
    {
        throw new Unhandled();
    }
}
