<?php
/**
 * @author Zoltán Zvara
 * @copyright Zoltán Zvara, 2014
 * @package Control
 */
namespace Arylium\Control;
/**
 * This exception should be inherited, when a control-based error
 * is being defined.
 *
 * This includes, for example, when an invalid request has called,
 * and there is no way to process it under the current circumstances.
 */
class Exception extends \Exception
{
}

/**
 * Abstract, inner routing mechanism.
 */
abstract class Router extends \Arylium\Component
{
    /**
     * Holds default routing keys in order.
     * @var array
     */
    static $Paths = ["Service", "Section"];
    /**
     * Holds predefined routes.
     * @var array
     */
    static $Routes = [];
    /**
     * Defines namespace mappings.
     * @var array
     */
    static $Mappings = [];
    /**
     * The service called (that is represented by a class).
     * @var string
     */
    protected $service;
    /**
     * The namespace created from the service name.
     * @var string
     */
    protected $namespace;
    /**
     * Name of the module the router belongs to.
     * @var string
     */
    protected $module;
    /**
     * This array holds the request parameters loaded from the URL.
     * @var Array
     */
    protected $Parameters = [];
    /**
     * Section requested inside the service.
     * @var string
     */
    protected $action;
    /**
     * Control object that represents the service.
     * @var Control
     */
    protected $Control;

    /**
     * Creates a new routing mechanism.
     * Loads and finds the requested service and its definitions.
     * @throws Exception
     * @returns Router
     */
    public function __construct()
    {
        parent::__construct();

        $path = explode("/", Request::getParameter($this->Settings["request_string"]));

        $this->service = array_shift($path);
        $this->action = array_shift($path);
        $this->Parameters = $path;

        // Searching the routing table for the requested service.
        if (array_key_exists($this->service, static::$Routes)) {
            $this->service = static::$Routes[$this->service];
        }

        // Building the path for the control component.
        // If the control has not been found, try to load the default service,
        //  or call the emergency method if everything fails.
        $control = $this->Settings["control_path"] . $this->service . "." . $this->Settings["control_extension"];
        if (!file_exists($control)) {
            $this->service = $this->Settings["default_service"];
            $control = $this->Settings["control_path"] . $this->service . "." . $this->Settings["control_extension"];
        }
        if (!file_exists($control)) {
            $this->whenLost();
        }

        // Define the namespace of the control component from the mapping table.
        if (array_key_exists($this->service, static::$Mappings)) {
            $this->namespace = static::$Mappings[$this->service];
        } else {
            $this->namespace = $this->service;
        }

        // Include the control component.
        include_once $control;
    }

    /**
     * @todo Validate settings.
     */
    protected function initialize()
    {
        parent::initialize();
    }

    /**
     * Serves the request by creating and calling the controller object
     * that belongs to the service.
     * @return $this Router
     */
    public function serve()
    {
        $control = ucfirst($this->namespace);
        if ($this->module) $control = $this->module . "\\" . $control;

        $this->beforeControl();
        /**
         * @var $Control Control
         */
        return new $control($this->Parameters, $this->service, $this->namespace, $this->action);
    }

    /**
     * Runs before the control gets created.
     */
    protected function beforeControl()
    {
    }

    /**
     * Called when the control component was not found in the routing process.
     */
    abstract protected function whenLost();
}