<?php
/**
 * @author Zoltán Zvara
 * @copyright Zoltán Zvara, 2014
 * @package Control
 * @version 1.0
 */
namespace Arylium;
/**
 * Holds site-specific information and a helper for controllers.
 */
class Site
{
    /**
     * Language object.
     * @var Language
     */
    private $Language;
    /**
     * Current namespace (that is requested by the client).
     * @var string
     */
    private $namespace;

    /**
     * Constructs a site-information object.
     * @param $namespace
     * @returns Site
     */
    public function __construct($namespace)
    {
        $this->namespace = $namespace;
        $this->Language = new Language();
    }

    /**
     * Gets the current language code.
     * @return string
     */
    public function getLanguageCode()
    {
        return $this->Language->getCode();
    }

    /**
     * Gets the language object.
     * @return Language
     */
    public function getLanguage()
    {
        return $this->Language;
    }

    /**
     * Gets the namespace.
     * @return string
     */
    public function getNamespace()
    {
        return $this->namespace;
    }
}

/**
 * Helper to identify the language requested.
 */
class Language
{
    const English = "en";
    const Hungarian = "hu";

    static $Languages = ["hu" => "hungarian",
        "en" => "english"];
    /**
     * Holds the base path (domain) of the server.
     * @var string
     */
    private $basePath;
    /**
     * Holds the requested server name (domain).
     * @var string
     */
    private $currentPath;
    /**
     * Default language to use, when not set.
     * @var string
     */
    private $defaultLanguage;
    /**
     * The language to use.
     * @var string
     */
    private $currentLanguage;

    /**
     * Creates and initializes the language object, also loading
     * language information.
     * @returns Language
     */
    public function __construct()
    {
        /**
         * Loading the pre-defined server base path.
         * If not set, the PHP variable named <i>SERVER_NAME</i> will be used;
         */
        if (defined('_serverBasePath')) {
            $this->basePath = _serverBasePath;
        } else {
            $this->basePath = $_SERVER["SERVER_NAME"];
        }
        /**
         * Current path has been set by <i>SERVER_NAME</i>.
         */
        $this->currentPath = $_SERVER["SERVER_NAME"];
        /**
         * Load the default language.
         * English will be the default if none set.
         */
        if (defined('_defaultLanguage')) {
            $this->defaultLanguage = _defaultLanguage;
        } else {
            $this->defaultLanguage = self::English;
        }

        $remainder = str_replace($this->basePath, "", $this->currentPath);
        $remainder = substr($remainder, 0, -1);
        foreach (self::$Languages as $language => $name) {
            if ($language == $remainder) {
                $this->currentLanguage = $remainder;
                break;
            }
        }
        if (!$this->currentLanguage) {
            $this->currentLanguage = $this->defaultLanguage;
        }

    }

    /**
     * Gets the current language code.
     * @return string
     */
    public function getCode()
    {
        return $this->currentLanguage;
    }
}